# OpenStack Cluster Installer (OCI)

OCI (OpenStack Cluster Installer) is a software to provision an OpenStack
cluster automatically. This package install a provisioning machine, which
consists of a DHCP server, a PXE boot server, and a puppet-master.

Once computers in the cluster boot for the first time, a Debian live system
is served by OCI, to act as a discovery image. This live system then reports
the hardware features back to OCI. The computers can then be installed with
Debian from that live system, configured with a puppet-agent that will connect
to the puppet-master of OCI. OpenStack is then provisionned in these machines.

## How to install your puppet-master/PXE server

### The package
The package is either available from plain Debian Sid/Buster, or from the
OpenStack stretch-queens repository:

```deb http://stretch-queens.debian.net/debian stretch-queens-backports main
deb-src http://stretch-queens.debian.net/debian stretch-queens-backports main

deb http://stretch-queens.debian.net/debian stretch-queens-backports-nochange main
deb-src http://stretch-queens.debian.net/debian stretch-queens-backports-nochange main```

The repository key is available this way:

```apt-get update
apt-get install --allow-unauthenticated -y openstack-backports-archive-keyring
apt-get update```

Then simply install the package:

```apt-get install openstack-cluster-installer```


### Configuring OCI
Configure it this way:

```a2ensite openstack-cluster-installer.conf
systemctl reload apache2
apt-get install openstack-pkg-tools
. /usr/share/openstack-pkg-tools/pkgos_func
PASSWORD=$(openssl rand -hex 16)
pkgos_inifile set /etc/openstack-cluster-installer/openstack-cluster-installer.conf database connection mysql+pymysql://oci:${PASSWORD}@localhost:3306/oci"
mysql --execute 'CREATE DATABASE oci;'
mysql --execute "GRANT ALL PRIVILEGES ON oci.* TO 'oci'@'localhost' IDENTIFIED BY '${PASSWORD}';"
apt-get install -y php-cli
cd /usr/share/openstack-cluster-installer ; php db_sync.php
chown www-data /etc/openstack-cluster-installer
chown www-data /etc/openstack-cluster-installer/openstack-cluster-installer.conf
cp -auxf /var/lib/openstack-cluster-installer/tftp/* /usr/share/openstack-cluster-installer
oci-root-ca-gen```

Then edit /etc/openstack-cluster-installer/openstack-cluster-installer.conf
and make it looks the way it pleases you (ie: change network values, etc.).

### ISC-DHCPD
Configure isc-dhcp to match your network configuration. Note that
"next-server" must be the address of your puppet-master node (ie: the dhcp
server that we're currently configuring).

Edit /etc/default/isc-dhcpd:

```sed -i 's/INTERFACESv4=.*/INTERFACESv4="eth0"/' /etc/default/isc-dhcp-server```

Then edit /etc/dhcp/dhcpd.conf:

```allow booting;
allow bootp;
default-lease-time 600;
max-lease-time 7200;
ddns-update-style none;
authoritative;
ignore-client-uids On;

subnet 192.168.100.0 netmask 255.255.255.0 {
        range 192.168.100.20 192.168.100.80;
        option domain-name infomaniak.ch;
        option domain-name-servers 9.9.9.9;
        option routers 192.168.100.1;
        option subnet-mask 255.255.255.0;
        option broadcast-address 192.168.100.255;
        next-server 192.168.100.2;
        if exists user-class and option user-class = "iPXE" {
                filename "http://192.168.100.2/oci/ipxe.php";
        } else {
                filename "pxelinux.0";
        }
}```

### tftpd
```sed -i 's#TFTP_DIRECTORY=.*#TFTP_DIRECTORY="/var/lib/openstack-cluster-installer/tftp"#' /etc/default/tftpd-hpa```

Then restart tftpd-hpa.

### Build OCI's live image ###
```mkdir -p /root/live-image
cd /root/live-image
openstack-cluster-installer-build-live-image --pxe-server-ip 192.168.100.2 --debian-mirror-addr http://deb.debian.org/debian --debian-security-mirror-addr http://security.debian.org/
cd ..
rm -rf /root/live-image```

### Configure puppet's ENC
```. /usr/share/openstack-pkg-tools/pkgos_func
pkgos_add_directive /etc/puppet/puppet.conf master "external_nodes = /usr/bin/oci-puppet-external-node-classifier" "# Path to enc"
pkgos_inifile set /etc/puppet/puppet.conf master external_nodes /usr/bin/oci-puppet-external-node-classifier
pkgos_add_directive /etc/puppet/puppet.conf master "node_terminus = exec" "# Tell what type of ENC"
pkgos_inifile set /etc/puppet/puppet.conf master node_terminus exec```

then restart puppet-master
